import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@ng-stack/forms';
import { Observable, Subject } from 'rxjs';
import { CountryModel } from 'src/app/modules/shared/models/country.model';
import { CountryService } from 'src/app/modules/shared/services/country/country.service';
import { map, startWith } from 'rxjs/operators';
import { AuthService } from 'src/app/modules/auth/services/auth/auth.service';

interface PhoneEntry{
    number: string;
    name: string;
    description: string;
    icon: string;
}
interface EmailEntry{
    number: string;
    name: string;
    description: string;
    icon: string;
}

class ProfileInformationBasicModel {
    name: string;
    surname: string;

    country: number;
    state: string;
    city: string;
    zip: string;
    phones: PhoneEntry[];
    emails: EmailEntry[];

}
@Component({
    selector: 'app-profile-edit-basics',
    templateUrl: './profile-edit-basics.component.html',
    styleUrls: ['./profile-edit-basics.component.scss']
})
export class ProfileEditBasicsComponent implements OnInit {

    form: FormGroup<ProfileInformationBasicModel>;
    onContentScroll = new Subject<number>();
    headerHeight = 200;
    placeholderBoxHeight = 0;
    filteredCountries: Observable<CountryModel[]>;
    countryInputChnage = new Subject<string>();
    countries: CountryModel[] = [];
    countryInput = '';

    constructor(
        private countryService: CountryService,
        authService: AuthService,
        formBuilder: FormBuilder,
    ) {
        const phones = formBuilder.array<PhoneEntry>([]);
        const emails = formBuilder.array<EmailEntry>([]);
        const user = authService.getUser();

        this.form = formBuilder.group<ProfileInformationBasicModel>({
            name: user.name,
            surname: user.surname,

            country: user.country,
            state: 'state',
            city: 'city',
            zip: 'zip',
            phones,
            emails,
        });
        window['form'] = this.form;

        this.countryService.getCountries().subscribe(countries => {
            this.countries = countries;
            this.countryInputChnage.next(this.countryInput);
            this.form.patchValue({
                country: this.form.value.country,
            });
        });
        this.filteredCountries = this.countryInputChnage.pipe(
          startWith(''),
          map(name => name ? this.filterCountries(name) : this.countries.slice())
        );
    }

    displayCountry = (countryId: number) => {
        const country = this.countries.find(i => i.id === countryId);
        return country ? country.name : '';
    }

    ngOnInit(): void {
    }

    scroll(e) {
        const scrollTop = e.target?.scrollTop;
        if (Number(scrollTop) === scrollTop) {
            this.placeholderBoxHeight = Math.min(scrollTop, this.headerHeight);
            this.onContentScroll.next(scrollTop);
        }
    }

    filterCountries(name: string) {
        return this.countries.filter(i => i.name.toLowerCase().includes(name));
    }

    countryFieldChanged(event) {
        this.countryInput = event.target.value.toLowerCase();
        this.countryInputChnage.next(this.countryInput);
    }
}
